/****************************************************************************
** File name    :  HTBusiDatabase.h
** Description  :  busi app database api
** Create date  :  2019.09.01
** Auther by    :  Liuyx
** Version info :  V1.0.01
** Copyright (C) 2002-2019 xi'an huatek, Inc Co., Ltd
** Update record:
** DATE         AUTHER   DESC
** -------------------------------------------------------------------------
** 2019.09.01   Liuyx    first build
****************************************************************************/
#ifndef __HT_BUSI_DATABASE_H
#define __HT_BUSI_DATABASE_H
#pragma warning(disable: 4819)
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif  /* __cplusplus */
#pragma pack (push ,1)


#pragma pack (pop)

    void vLoadCommunicationConf();

    // 添加Ied不能连接的告警消息
    void vSetIedDevicesWarnMesg(char *Ied_ip, int port, time_t altime);
    // 删除Ied服务不能连接的告警消息
    void vDelIedDevicesWarnMesg(char *Ied_ip, int port, time_t altime);

    // 加载传感器基本信息
    void vLoadBaseDeviceInfo();

    // 加载通信参数配置
    void vLoadCommunicationConf();

    // 加载控制参数
    void vLoadDeviceCtrlParam();

    // 加载ICD配置节点与传感器绑定关系
    void vLoadICDParameterBind();

    // 加载阈值配置参数
    void vLoadCenterDeviceThreshold();
    void vLoadNoiseDeviceThreshold();
    void vLoadAmpliDeviceThreshold();

    // 显示阈值配置信息
    void vPrtDeviceThresholdConf();
    // 显示传感器控制参数
    void vPrtDeviceCtrlParam();
    // 显示传感器控制参数
    void vPrtICDParameterBind();

    // 插入数据
    void vSetRealDevicesMoniMesg(ST_DB_BODY *pData);
    // 添加传感器上传的告警消息
    void vSetRealDevicesWarnMesg(ST_DB_WARN &stWarn);

#ifdef  __cplusplus
}
#endif

#endif /* end __HT_BUSI_DATABASE_H */

